#!/bin/bash
#

# need an in source build

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e6:p278|e6:p279|e7:p279|e7:p2710|e9:p2710> <debug|opt|prof> [tar]"
   echo "        p2710 -> python v2_7_10"
   echo "        p279  -> python v2_7_9"
   echo "        p278  -> python v2_7_8"
   echo "        e9 -> gcc v4_9_3"
   echo "        e7 -> gcc v4_9_2"
   echo "        e6 -> gcc v4_9_1"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
    flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=ifdhc
origpkgver=v1_8_7
pkgver=${origpkgver}
ssibuildshims_version=v0_09_00
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgtarfile=${package}-${origpkgver}.tar 

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

cd ${pkgdir} 
tar xf ${tardir}/${pkgtarfile} || die "ERROR: failed to unwind ${pkgtarfile}"
patch -b -p0 < "${patchdir}/ifdhc.patch" || die "ERROR: application of patch ${patchdir}/ifdhc.patch failed."

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

echo "PYTHONHOME is ${PYTHONHOME}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

case ${basequal} in
e6:p278) stdcxx="-std=c++1y" ;;
e6:p279) stdcxx="-std=c++1y" ;;
e7:p279) stdcxx="-std=c++1y" ;;
e7:p2710) stdcxx="-std=c++1y" ;;
e9:p2710) stdcxx="-std=c++1y" ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

case ${extraqual} in
debug) ARCH="${stdcxx} -O0 -g";;
prof)  ARCH="${stdcxx} -g -p -pg";;
opt)   ARCH="${stdcxx} -O2";;
esac
export ARCH

export PYTHON_CONF=`echo $PYTHON_LIB/python*/config`

if [ `type swig >& /dev/null` ]
then

swig_minor=`swig -version | grep Version | cut -f 3 -d " " | cut -f 2 -d "."`
swig_patch=`swig -version | grep Version | cut -f 3 -d " " | cut -f 3 -d "."`

if [ ${swig_minor} -eq 3 ] && [ ${swig_patch} -ge 40 ]
then
 mv ${pkgdir}/ifdh/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch.orig
 cp -p ${patchdir}/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch
elif [ ${swig_minor} -gt 3 ]
then
 mv ${pkgdir}/ifdh/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch.orig
 cp -p ${patchdir}/ifdh_wrap.patch ${pkgdir}/ifdh/ifdh_wrap.patch
fi

else

  echo "swig is not available"

fi

set -x

cd ${pkgdir} 

make IFDHC_VERSION=${origpkgver} all || die "ERROR: make all failed"
make install || die "ERROR: make install failed"
make clean || die "ERROR: make clean failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
echo "unsetup"
unsetup ${package}
echo "setup"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
